@@ -19,7 +19,7 @@ from __future__ import division
 from __future__ import print_function
 from __future__ import unicode_literals
 
-__version__ = '0.0.39'
+__version__ = '0.0.43'
 
 # Used with pypi checks and other messages related to pip
 _wandb_module = 'wandb-ng'
@@ -44,8 +44,10 @@ setup = wandb_sdk.setup
 save = wandb_sdk.save
 watch = wandb_sdk.watch
 login = wandb_sdk.login
+helper = wandb_sdk.helper
 Artifact = wandb_sdk.Artifact
 Settings = wandb_sdk.Settings
+Config = wandb_sdk.Config
 
 from wandb.apis import InternalApi, PublicApi
 from wandb.errors.error import CommError
@@ -75,6 +77,7 @@ from wandb import superagent
 # from wandb.core import *
 from wandb.viz import visualize
 from wandb import plots
+from wandb.integration.sagemaker import sagemaker_auth
 
 
 # Used to make sure we don't use some code in the incorrect process context
@@ -126,13 +129,14 @@ log_artifact = _preinit.PreInitCallable(
 # record of patched libraries
 patched = {"tensorboard": [], "keras": [], "gym": []}
 
-keras = _lazyloader.LazyLoader("wandb.keras", globals(), "wandb.framework.keras")
+keras = _lazyloader.LazyLoader("wandb.keras", globals(), "wandb.integration.keras")
 sklearn = _lazyloader.LazyLoader("wandb.sklearn", globals(), "wandb.sklearn")
-tensorflow = _lazyloader.LazyLoader("wandb.tensorflow", globals(), "wandb.tensorflow")
-xgboost = _lazyloader.LazyLoader("wandb.xgboost", globals(), "wandb.framework.xgboost")
-gym = _lazyloader.LazyLoader("wandb.gym", globals(), "wandb.framework.gym")
+tensorflow = _lazyloader.LazyLoader("wandb.tensorflow", globals(), "wandb.integration.tensorflow")
+xgboost = _lazyloader.LazyLoader("wandb.xgboost", globals(), "wandb.integration.xgboost")
+tensorboard = _lazyloader.LazyLoader("wandb.tensorboard", globals(), "wandb.integration.tensorboard")
+gym = _lazyloader.LazyLoader("wandb.gym", globals(), "wandb.integration.gym")
 lightgbm = _lazyloader.LazyLoader(
-    "wandb.lightgbm", globals(), "wandb.framework.lightgbm"
+    "wandb.lightgbm", globals(), "wandb.integration.lightgbm"
 )
 docker = _lazyloader.LazyLoader("wandb.docker", globals(), "wandb.docker")
 jupyter = _lazyloader.LazyLoader("wandb.jupyter", globals(), "wandb.jupyter")
