@@ -29,12 +29,13 @@ import wandb
 from wandb import __version__
 from wandb.old.core import wandb_dir, Error
 from wandb import env
-#from wandb.git_repo import GitRepo
 from wandb.old.settings import Settings
 from wandb.old import retry
 from wandb import util
 from wandb.apis.normalize import normalize_exceptions
 from wandb.errors.error import CommError, UsageError
+from wandb.lib.filenames import DIFF_FNAME, METADATA_FNAME
+from wandb.lib.git import GitRepo
 
 from .file_stream import FileStreamApi
 from .progress import Progress
@@ -72,8 +73,7 @@ class Api(object):
         self.default_settings.update(default_settings or {})
         self.retry_uploads = 10
         self._settings = Settings(load_settings=load_settings)
-        #self.git = GitRepo(remote=self.settings("git_remote"))
-        self.git = None
+        self.git = GitRepo(remote=self.settings("git_remote"))
         # Mutable settings set by the _file_stream_api
         self.dynamic_settings = {
             'system_sample_seconds': 2,
@@ -128,6 +128,9 @@ class Api(object):
 
         if 'errors' in data and isinstance(data['errors'], list):
             for err in data['errors']:
+                # Our tests and potentially some api endpoints return a string error?
+                if isinstance(err, six.string_types):
+                    err = {"message": err}
                 if not err.get('message'):
                     continue
                 wandb.termerror('Error while calling W&B API: {} ({})'.format(err['message'], res))
@@ -139,7 +142,7 @@ class Api(object):
         """Syncs this run with spell"""
         try:
             env = env or os.environ
-            run.config._set_wandb("spell_url", env.get("SPELL_RUN_URL"))
+            run.config["_wandb"]["spell_url"] = env.get("SPELL_RUN_URL")
             run.config.persist()
             try:
                 url = run.get_url()
@@ -161,7 +164,7 @@ class Api(object):
         to history editing as long as the user never does "push -f" to break
         history on an upstream branch.
 
-        Writes the first patch to <out_dir>/diff.patch and the second to
+        Writes the first patch to <out_dir>/<DIFF_FNAME> and the second to
         <out_dir>/upstream_diff_<commit_id>.patch.
 
         Args:
@@ -173,7 +176,7 @@ class Api(object):
         try:
             root = self.git.root
             if self.git.dirty:
-                patch_path = os.path.join(out_dir, 'diff.patch')
+                patch_path = os.path.join(out_dir, DIFF_FNAME)
                 if self.git.has_submodule_diff:
                     with open(patch_path, 'wb') as patch:
                         # we diff against HEAD to ensure we get changes in the index
@@ -223,9 +226,8 @@ class Api(object):
             key = auth[-1]
         # Environment should take precedence
         env_key = self._environ.get(env.API_KEY)
-        if env_key is not None:
-            key = env_key
-        return key
+        default_key = self.default_settings.get("api_key")
+        return env_key or key or default_key
 
     @property
     def api_url(self):
@@ -541,7 +543,7 @@ class Api(object):
                     files(names: ["wandb-metadata.json"]) {
                         edges {
                             node {
-                                url
+                                directUrl
                             }
                         }
                     }
@@ -554,13 +556,13 @@ class Api(object):
             'name': project, 'run': run, 'entity': entity
         })
         if response['model'] == None:
-            raise ValueError("Run {}/{}/{} not found".format(entity, project, run) )
+            raise CommError("Run {}/{}/{} not found".format(entity, project, run) )
         run = response['model']['bucket']
         commit = run['commit']
         patch = run['patch']
         config = json.loads(run['config'] or '{}')
         if len(run['files']['edges']) > 0:
-            url = run['files']['edges'][0]['node']['url']
+            url = run['files']['edges'][0]['node']['directUrl']
             res = requests.get(url)
             res.raise_for_status()
             metadata = res.json()
@@ -573,9 +575,9 @@ class Api(object):
         """Check if a run exists and get resume information.
 
         Args:
-            entity (str, optional): The entity to scope this project to.
+            entity (str): The entity to scope this project to.
             project_name (str): The project to download, (can include bucket)
-            run (str, optional): The run to download
+            run (str): The run to download
         """
         query = gql('''
         query Model($project: String!, $entity: String, $name: String!) {
@@ -757,6 +759,7 @@ class Api(object):
                     displayName
                     description
                     config
+                    sweepName
                     project {
                         id
                         name
@@ -766,6 +769,7 @@ class Api(object):
                         }
                     }
                 }
+                inserted
             }
         }
         ''')
@@ -800,7 +804,7 @@ class Api(object):
             if entity:
                 self.set_setting('entity', entity['name'])
 
-        return response['upsertBucket']['bucket']
+        return response['upsertBucket']['bucket'], response['upsertBucket']['inserted']
 
     @normalize_exceptions
     def upload_urls(self, project, files, run=None, entity=None, description=None):
@@ -1000,15 +1004,19 @@ class Api(object):
             response.raise_for_status()
         except requests.exceptions.RequestException as e:
             status_code = e.response.status_code if e.response != None else 0
+            # We need to rewind the file for the next retry (the file passed in is seeked to 0)
+            progress.rewind()
             # Retry errors from cloud storage or local network issues
-            if status_code in (308, 409, 429, 500, 502, 503, 504) or isinstance(e, (requests.exceptions.Timeout, requests.exceptions.ConnectionError)):
+            if status_code in (308, 408, 409, 429, 500, 502, 503, 504) or isinstance(e, (requests.exceptions.Timeout, requests.exceptions.ConnectionError)):
                 util.sentry_reraise(retry.TransientException(exc=e))
             else:
                 util.sentry_reraise(e)
 
         return response
 
-    upload_file_retry = normalize_exceptions(retry.retriable(num_retries=5)(upload_file))
+    # This Retry class is initialized once for each Api instance, so this
+    # defaults to retrying 1 million times per process or 365 days
+    upload_file_retry = normalize_exceptions(retry.retriable()(upload_file))
 
     @normalize_exceptions
     def register_agent(self, host, sweep_id=None, project_name=None, entity=None):
@@ -1303,15 +1311,6 @@ class Api(object):
             open_file.close()
         return responses
 
-    def get_file_stream_api(self):
-        """This creates a new file pusher thread.  Call start to initiate the thread that talks to W&B"""
-        if not self._file_stream_api:
-            if self._current_run_id is None:
-                raise UsageError(
-                    'Must have a current run to use file stream API.')
-            self._file_stream_api = FileStreamApi(self, self._current_run_id)
-        return self._file_stream_api
-
     def use_artifact(self, artifact_id, entity_name=None, project_name=None, run_name=None):
         query = gql('''
         mutation UseArtifact(
@@ -1418,6 +1417,13 @@ class Api(object):
                         artifactCollectionName
                         alias
                     }
+                    artifactSequence {
+                        id
+                        latestArtifact {
+                            id
+                            versionIndex
+                        }
+                    }
                 }
             }
         }
@@ -1446,7 +1452,8 @@ class Api(object):
         for alias in av["aliases"]:
             if alias["artifactCollectionName"] == artifact_collection_name and re.match(r"^v\d+$", alias["alias"]):
                 av['version'] = alias["alias"]
-        return av
+        latest = response['createArtifact']['artifact']['artifactSequence'].get('latestArtifact')
+        return av, latest
 
     def commit_artifact(self, artifact_id):
         mutation = gql('''
@@ -1468,20 +1475,24 @@ class Api(object):
         })
         return response
 
-    def create_artifact_manifest(self, name, digest, artifact_id, entity=None, project=None, run=None):
+    def create_artifact_manifest(self, name, digest, artifact_id,
+                                 base_artifact_id=None,entity=None, project=None, run=None, include_upload=True):
         mutation = gql('''
         mutation CreateArtifactManifest(
             $name: String!,
             $digest: String!,
             $artifactID: ID!,
+            $baseArtifactID: ID,
             $entityName: String!,
             $projectName: String!,
-            $runName: String!
+            $runName: String!,
+            $includeUpload: Boolean!
         ) {
             createArtifactManifest(input: {
                 name: $name,
                 digest: $digest,
                 artifactID: $artifactID,
+                baseArtifactID: $baseArtifactID,
                 entityName: $entityName,
                 projectName: $projectName,
                 runName: $runName
@@ -1492,8 +1503,8 @@ class Api(object):
                         id
                         name
                         displayName
-                        uploadUrl
-                        uploadHeaders
+                        uploadUrl @include(if: $includeUpload)
+                        uploadHeaders @include(if: $includeUpload)
                     }
                 }
             }
@@ -1508,9 +1519,11 @@ class Api(object):
             'name': name,
             'digest': digest,
             'artifactID': artifact_id,
+            'baseArtifactID': base_artifact_id,
             'entityName': entity_name,
             'projectName': project_name,
             'runName': run_name,
+            'includeUpload': include_upload,
         })
 
         return response['createArtifactManifest']['artifactManifest']['file']
@@ -1519,10 +1532,12 @@ class Api(object):
     def create_artifact_files(self, artifact_files):
         mutation = gql('''
         mutation CreateArtifactFiles(
+            $storageLayout: ArtifactStorageLayout!
             $artifactFiles: [CreateArtifactFileSpecInput!]!
         ) {
             createArtifactFiles(input: {
-                artifactFiles: $artifactFiles
+                artifactFiles: $artifactFiles,
+                storageLayout: $storageLayout
             }) {
                 files {
                     edges {
@@ -1532,6 +1547,9 @@ class Api(object):
                             displayName
                             uploadUrl
                             uploadHeaders
+                            artifact {
+                                id
+                            }
                         }
                     }
                 }
@@ -1539,7 +1557,15 @@ class Api(object):
         }
         ''')
 
+        # TODO: we should use constants here from interface/artifacts.py
+        # but probably don't want the dependency. We're going to remove
+        # this setting in a future release, so I'm just hard-coding the strings.
+        storage_layout = 'V2'
+        if env.get_use_v1_artifacts():
+            storage_layout = 'V1'
+
         response = self.gql(mutation, variable_values={
+            'storageLayout': storage_layout,
             'artifactFiles': [af for af in artifact_files]
         })
 
