@@ -1,3 +1,4 @@
+#
 # -*- coding: utf-8 -*-
 """Run - Run object.
 
@@ -12,19 +13,25 @@ import collections
 import glob
 import json
 import logging
+import numbers
 import os
 import platform
 import sys
+import threading
+import time
 import traceback
 
 import click
-from six import string_types
+from six import iteritems, string_types
+from six.moves import _thread as thread
 from six.moves.urllib.parse import quote as url_quote
 import wandb
+from wandb import trigger
 from wandb.apis import internal, public
 from wandb.data_types import _datatypes_set_callback
 from wandb.errors import Error
-from wandb.lib import module, redirect
+from wandb.interface.summary_record import SummaryRecord
+from wandb.lib import filenames, module, proto_util, redirect, sparkline
 from wandb.util import sentry_set_scope, to_forward_slash_path
 from wandb.viz import Visualize
 
@@ -35,8 +42,8 @@ from . import wandb_summary
 if wandb.TYPE_CHECKING:  # type: ignore
     from typing import Optional
 
-
 logger = logging.getLogger("wandb")
+EXIT_TIMEOUT = 60
 
 
 class Run(object):
@@ -80,24 +87,62 @@ class ExitHooks(object):
         if self.was_ctrl_c():
             self.exit_code = 255
 
-        print("except handle")
         traceback.print_exception(exc_type, exc, *tb)
 
 
+class RunStatusChecker(object):
+    """Periodically polls the background process for relevant updates.
+
+    For now, we just use this to figure out if the user has requested a stop.
+    """
+
+    def __init__(self, interface, polling_interval=15):
+        self._interface = interface
+        self._polling_interval = polling_interval
+
+        self._join_event = threading.Event()
+        self._thread = threading.Thread(target=self.check_status)
+        self._thread.daemon = True
+        self._thread.start()
+
+    def check_status(self):
+        join_requested = False
+        while not join_requested:
+            status_response = (
+                # 'or False' because this could return None.
+                self._interface.communicate_status(check_stop_req=True)
+                or False
+            )
+            if status_response.run_should_stop:
+                thread.interrupt_main()
+                return
+            join_requested = self._join_event.wait(self._polling_interval)
+
+    def stop(self):
+        self._join_event.set()
+
+    def join(self):
+        self.stop()
+        self._thread.join()
+
+
 class RunManaged(Run):
     def __init__(self, config=None, settings=None):
         self._config = wandb_config.Config()
         self._config._set_callback(self._config_callback)
-        self.summary = wandb_summary.Summary()
-        self.summary._set_callback(self._summary_callback)
-        self.history = wandb_history.History()
+        self._config._set_settings(settings)
+        self._backend = None
+        self.summary = wandb_summary.Summary(
+            self._summary_get_current_summary_callback,
+        )
+        self.summary._set_update_callback(self._summary_update_callback)
+        self.history = wandb_history.History(self)
         self.history._set_callback(self._history_callback)
 
         _datatypes_set_callback(self._datatypes_callback)
 
         self._settings = settings
         self._wl = None
-        self._backend = None
         self._reporter = None
         self._data = dict()
 
@@ -106,11 +151,14 @@ class RunManaged(Run):
         self._group = None
         self._job_type = None
         self._run_id = settings.run_id
+        self._start_time = time.time()
+        self._starting_step = 0
         self._name = None
         self._notes = None
         self._tags = None
 
         self._hooks = None
+        self._teardown_hooks = []
         self._redirect_cb = None
         self._out_redir = None
         self._err_redir = None
@@ -121,6 +169,11 @@ class RunManaged(Run):
         self._stdout_slave_fd = None
         self._stderr_slave_fd = None
         self._exit_code = None
+        self._exit_result = None
+        self._final_summary = None
+        self._sampled_history = None
+
+        self._output_writer = None
 
         # Pull info from settings
         self._init_from_settings(settings)
@@ -129,13 +182,32 @@ class RunManaged(Run):
         # actual run comes back.
         sentry_set_scope("user", self._entity, self._project)
 
-        # Returned from backend send_run_sync, set from wandb_init?
+        # Returned from backend request_run(), set from wandb_init?
         self._run_obj = None
+        self._run_obj_offline = None
+
+        # Created when the run "starts".
+        self._run_status_checker = None
+
+        self._poll_exit_response = None
 
         config = config or dict()
         wandb_key = "_wandb"
         config.setdefault(wandb_key, dict())
-        config[wandb_key]["cli_version"] = wandb.__version__
+
+        wandb_data = dict()
+        wandb_data["cli_version"] = wandb.__version__
+        wandb_data["python_version"] = platform.python_version()
+        wandb_data["is_jupyter_run"] = settings._jupyter or False
+        wandb_data["is_kaggle_kernel"] = settings._kaggle or False
+        hf_version = huggingface_version()
+        if hf_version:
+            wandb_data["huggingface_version"] = hf_version
+        framework = self._telemetry_get_framework()
+        if framework:
+            wandb_data["framework"] = framework
+        config[wandb_key].update(wandb_data)
+
         if settings.save_code and settings.program_relpath:
             config[wandb_key]["code_path"] = to_forward_slash_path(
                 os.path.join("code", settings.program_relpath)
@@ -143,6 +215,33 @@ class RunManaged(Run):
         self._config._update(config)
         self._atexit_cleanup_called = None
         self._use_redirect = True
+        self._progress_step = 0
+
+    def _telemetry_get_framework(self):
+        """Get telemetry data for internal config structure."""
+        # detect framework by checking what is loaded
+        loaded = {}
+        loaded["lightgbm"] = sys.modules.get("lightgbm")
+        loaded["catboost"] = sys.modules.get("catboost")
+        loaded["xgboost"] = sys.modules.get("xgboost")
+        loaded["fastai"] = sys.modules.get("fastai")
+        loaded["torch"] = sys.modules.get("torch")
+        loaded["keras"] = sys.modules.get("keras")  # vanilla keras
+        loaded["tensorflow"] = sys.modules.get("tensorflow")
+        loaded["sklearn"] = sys.modules.get("sklearn")
+
+        priority = (
+            "lightgbm",
+            "catboost",
+            "xgboost",
+            "fastai",
+            "torch",
+            "keras",
+            "tensorflow",
+            "sklearn",
+        )
+        framework = next((f for f in priority if loaded.get(f)), None)
+        return framework
 
     def _init_from_settings(self, settings):
         if settings.entity is not None:
@@ -151,8 +250,8 @@ class RunManaged(Run):
             self._project = settings.project
         if settings.run_group is not None:
             self._group = settings.run_group
-        if settings.job_type is not None:
-            self._job_type = settings.job_type
+        if settings.run_job_type is not None:
+            self._job_type = settings.run_job_type
         if settings.run_name is not None:
             self._name = settings.run_name
         if settings.run_notes is not None:
@@ -179,6 +278,8 @@ class RunManaged(Run):
         if self._tags is not None:
             for tag in self._tags:
                 run.tags.append(tag)
+        if self._start_time is not None:
+            run.start_time.FromSeconds(int(self._start_time))
         # Note: run.config is set in interface/interface:_make_run()
 
     def __getstate__(self):
@@ -197,6 +298,8 @@ class RunManaged(Run):
 
     @property
     def name(self):
+        if self._name:
+            return self._name
         if not self._run_obj:
             return None
         return self._run_obj.display_name
@@ -205,10 +308,12 @@ class RunManaged(Run):
     def name(self, name):
         self._name = name
         if self._backend:
-            self._backend.interface.send_run(self)
+            self._backend.interface.publish_run(self)
 
     @property
     def notes(self):
+        if self._notes:
+            return self._notes
         if not self._run_obj:
             return None
         return self._run_obj.notes
@@ -217,7 +322,21 @@ class RunManaged(Run):
     def notes(self, notes):
         self._notes = notes
         if self._backend:
-            self._backend.interface.send_run(self)
+            self._backend.interface.publish_run(self)
+
+    @property
+    def tags(self):
+        if self._tags:
+            return self._tags
+        if not self._run_obj:
+            return None
+        return self._run_obj.tags
+
+    @tags.setter
+    def tags(self, tags):
+        self._tags = tags
+        if self._backend:
+            self._backend.interface.publish_run(self)
 
     @property
     def id(self):
@@ -231,11 +350,41 @@ class RunManaged(Run):
                 parts.append(e)
         return "/".join(parts)
 
-    def project_name(self, api=None):
+    @property
+    def start_time(self):
         if not self._run_obj:
-            wandb.termwarn("Project name not available in offline run")
+            return self._start_time
+        else:
+            return self._run_obj.start_time.ToSeconds()
+
+    @property
+    def starting_step(self):
+        if not self._run_obj:
+            return self._starting_step
+        else:
+            return self._run_obj.starting_step
+
+    @property
+    def resumed(self):
+        return self._starting_step > 0
+
+    @property
+    def step(self):
+        return self.history._step
+
+    def project_name(self, api=None):
+        run_obj = self._run_obj or self._run_obj_offline
+        return run_obj.project
+
+    def get_url(self):
+        if not self._run_obj:
+            wandb.termwarn("URL not available in offline run")
             return
-        return self._run_obj.project
+        return self._get_run_url()
+
+    @property
+    def url(self):
+        return self.get_url()
 
     @property
     def entity(self):
@@ -262,14 +411,18 @@ class RunManaged(Run):
 
     def _config_callback(self, key=None, val=None, data=None):
         logger.info("config_cb %s %s %s", key, val, data)
-        self._backend.interface.send_config(data)
+        self._backend.interface.publish_config(data)
 
-    def _summary_callback(self, key=None, val=None, data=None):
-        self._backend.interface.send_summary(data)
+    def _summary_update_callback(self, summary_record: SummaryRecord):
+        self._backend.interface.publish_summary(summary_record)
+
+    def _summary_get_current_summary_callback(self):
+        ret = self._backend.interface.request_summary()
+        return proto_util.dict_from_proto_list(ret.item)
 
     def _datatypes_callback(self, fname):
         files = dict(files=[(fname, "now")])
-        self._backend.interface.send_files(files)
+        self._backend.interface.publish_files(files)
 
     def _history_callback(self, row=None, step=None):
 
@@ -288,12 +441,16 @@ class RunManaged(Run):
         if visualize_persist_config:
             self._config_callback(data=self._config._as_dict())
 
-        self._backend.interface.send_history(row, step)
-        self.summary.update(row)
+        self._backend.interface.publish_history(row, step)
 
     def _console_callback(self, name, data):
-        logger.info("callback: %s, %s", name, data)
-        self._backend.interface.send_output(name, data)
+        # logger.info("console callback: %s, %s", name, data)
+        self._backend.interface.publish_output(name, data)
+
+    def _tensorboard_callback(self, logdir, save=None):
+        logger.info("tensorboard callback: %s, %s", logdir, save)
+        save = True if save is None else save
+        self._backend.interface.publish_tbdata(logdir, save)
 
     def _set_library(self, library):
         self._wl = library
@@ -304,11 +461,19 @@ class RunManaged(Run):
     def _set_reporter(self, reporter):
         self._reporter = reporter
 
+    def _set_teardown_hooks(self, hooks):
+        self._teardown_hooks = hooks
+
     def _set_run_obj(self, run_obj):
         self._run_obj = run_obj
+        # TODO: Update run summary when resuming?
+        self.history._update_step()
         # TODO: It feels weird to call this twice..
         sentry_set_scope("user", run_obj.entity, run_obj.project, self._get_run_url())
 
+    def _set_run_obj_offline(self, run_obj):
+        self._run_obj_offline = run_obj
+
     def _add_singleton(self, type, key, value):
         """Stores a singleton item to wandb config.
 
@@ -465,7 +630,6 @@ class RunManaged(Run):
                 self.history._step = step
         elif commit is None:
             commit = True
-        #  TODO: ensure history is pushed on exit for non-added rows
         if commit:
             self.history._row_add(data)
         else:
@@ -554,7 +718,7 @@ class RunManaged(Run):
                 % file_str
             )
         files_dict = dict(files=[(wandb_glob_str, policy)])
-        self._backend.interface.send_files(files_dict)
+        self._backend.interface.publish_files(files_dict)
         return files
 
     def restore(
@@ -599,6 +763,10 @@ class RunManaged(Run):
         used when creating multiple runs in the same process.  We automatically
         call this method when your script exits.
         """
+        # detach logger, other setup cleanup
+        logger.info("joining run %s", self.path)
+        for hook in self._teardown_hooks:
+            hook()
         self._atexit_cleanup(exit_code=exit_code)
         if len(self._wl._global_run_stack) > 0:
             self._wl._global_run_stack.pop()
@@ -620,53 +788,129 @@ class RunManaged(Run):
         )
         return url
 
+    def _get_sweep_url(self):
+        """Generate a url for a sweep.
+
+        Returns:
+            string - url if the run is part of a sweep
+            None - if the run is not part of the sweep
+        """
+
+        r = self._run_obj
+        sweep_id = r.sweep_id
+        if not sweep_id:
+            return
+
+        app_url = self._settings.base_url.replace("//api.", "//app.")
+
+        return "{base}/{entity}/{project}/sweeps/{sweepid}".format(
+            base=app_url,
+            entity=url_quote(r.entity),
+            project=url_quote(r.project),
+            sweepid=url_quote(sweep_id),
+        )
+
     def _get_run_name(self):
         r = self._run_obj
         return r.display_name
 
     def _display_run(self):
-        emojis = dict(star="", broom="", rocket="")
-        if platform.system() != "Windows":
-            emojis = dict(star="⭐️", broom="🧹", rocket="🚀")
         project_url = self._get_project_url()
         run_url = self._get_run_url()
-        wandb.termlog(
-            "{} View project at {}".format(
-                emojis.get("star", ""),
-                click.style(project_url, underline=True, fg="blue"),
+        sweep_url = self._get_sweep_url()
+        if self._settings._jupyter:
+            from IPython.core.display import display, HTML  # type: ignore
+
+            sweep_line = (
+                'Sweep page: <a href="{}" target="_blank">{}</a><br/>\n'.format(
+                    sweep_url, sweep_url
+                )
+                if sweep_url
+                else ""
             )
-        )
-        wandb.termlog(
-            "{} View run at {}".format(
-                emojis.get("rocket", ""),
-                click.style(run_url, underline=True, fg="blue"),
+            docs_html = '<a href="https://docs.wandb.com/integrations/jupyter.html" target="_blank">(Documentation)</a>'  # noqa: E501
+            display(
+                HTML(
+                    """
+                Logging results to <a href="https://wandb.com" target="_blank">Weights & Biases</a> {}.<br/>
+                Project page: <a href="{}" target="_blank">{}</a><br/>
+                {}Run page: <a href="{}" target="_blank">{}</a><br/>
+            """.format(  # noqa: E501
+                        docs_html,
+                        project_url,
+                        project_url,
+                        sweep_line,
+                        run_url,
+                        run_url,
+                    )
+                )
             )
-        )
+        else:
+            emojis = dict(star="", broom="", rocket="")
+            if platform.system() != "Windows":
+                emojis = dict(star="⭐️", broom="🧹", rocket="🚀")
+
+            wandb.termlog(
+                "{} View project at {}".format(
+                    emojis.get("star", ""),
+                    click.style(project_url, underline=True, fg="blue"),
+                )
+            )
+            if sweep_url:
+                wandb.termlog(
+                    "{} View sweep at {}".format(
+                        emojis.get("broom", ""),
+                        click.style(sweep_url, underline=True, fg="blue"),
+                    )
+                )
+            wandb.termlog(
+                "{} View run at {}".format(
+                    emojis.get("rocket", ""),
+                    click.style(run_url, underline=True, fg="blue"),
+                )
+            )
+            if not self._settings._offline:
+                wandb.termlog("Run `wandb off` to turn off syncing.")
 
     def _redirect(self, stdout_slave_fd, stderr_slave_fd):
-        console = self._settings.console
+        console = self._settings._console
         logger.info("redirect: %s", console)
 
-        if console == "redirect":
-            logger.info("redirect1")
-            out_cap = redirect.Capture(name="stdout", cb=self._redirect_cb)
+        if console == self._settings.Console.REDIRECT:
+            logger.info("Redirecting console.")
+            out_cap = redirect.Capture(
+                name="stdout", cb=self._redirect_cb, output_writer=self._output_writer
+            )
+            err_cap = redirect.Capture(
+                name="stderr", cb=self._redirect_cb, output_writer=self._output_writer
+            )
             out_redir = redirect.Redirect(
                 src="stdout", dest=out_cap, unbuffered=True, tee=True
             )
-            err_cap = redirect.Capture(name="stderr", cb=self._redirect_cb)
             err_redir = redirect.Redirect(
                 src="stderr", dest=err_cap, unbuffered=True, tee=True
             )
-            try:
-                out_redir.install()
-                err_redir.install()
-                self._out_redir = out_redir
-                self._err_redir = err_redir
-                logger.info("redirect2")
-            except (OSError, AttributeError) as e:
-                logger.error("failed to redirect", exc_info=e)
+        elif console == self._settings.Console.NOTEBOOK:
+            logger.info("Redirecting notebook output.")
+            out_redir = redirect.StreamWrapper(
+                name="stdout", cb=self._redirect_cb, output_writer=self._output_writer
+            )
+            err_redir = redirect.StreamWrapper(
+                name="stderr", cb=self._redirect_cb, output_writer=self._output_writer
+            )
+        elif console == self._settings.Console.OFF:
             return
-
+        else:
+            raise ValueError("unhandled console")
+        try:
+            out_redir.install()
+            err_redir.install()
+            self._out_redir = out_redir
+            self._err_redir = err_redir
+            logger.info("Redirects installed.")
+        except Exception as e:
+            print(e)
+            logger.error("Failed to redirect.", exc_info=e)
         return
 
         # TODO(jhr): everything below here is not executed as we only support redir mode
@@ -720,9 +964,24 @@ class RunManaged(Run):
 
         exit_code = exit_code or self._hooks.exit_code if self._hooks else 0
         logger.info("got exitcode: %d", exit_code)
+        if exit_code == 0:
+            # Cleanup our resume file on a clean exit
+            if os.path.exists(self._settings.resume_fname):
+                os.remove(self._settings.resume_fname)
 
         self._exit_code = exit_code
-        self._on_finish()
+        try:
+            self._on_finish()
+        except KeyboardInterrupt:
+            wandb.termerror("Control-C detected -- Run data was not synced")
+            os._exit(-1)
+        except Exception as e:
+            self._console_stop()
+            self._backend.cleanup()
+            logger.error("Problem finishing run", exc_info=e)
+            wandb.termerror("Problem finishing run")
+            traceback.print_exception(*sys.exc_info())
+            os._exit(-1)
         self._on_final()
 
     def _console_start(self):
@@ -735,38 +994,132 @@ class RunManaged(Run):
             # setup fake callback
             self._redirect_cb = self._console_callback
 
+        output_log_path = os.path.join(self.dir, filenames.OUTPUT_FNAME)
+        self._output_writer = WriteSerializingFile(open(output_log_path, "wb"))
         self._redirect(self._stdout_slave_fd, self._stderr_slave_fd)
-        pass
 
     def _console_stop(self):
         self._restore()
+        self._output_writer.close()
+        self._output_writer = None
 
     def _on_start(self):
+        if self._settings._offline:
+            wandb.termlog("Offline run mode, not syncing to the cloud.")
         wandb.termlog("Tracking run with wandb version {}".format(wandb.__version__))
+        if self._settings._offline:
+            wandb.termlog(
+                (
+                    "W&B is disabled in this directory.  "
+                    "Run `wandb on` to enable cloud syncing."
+                )
+            )
+        wandb.termlog(
+            "Run data is saved locally in {}".format(self._settings._sync_dir)
+        )
         if self._run_obj:
-            run_state_str = "Syncing run"
+            if self.resumed:
+                run_state_str = "Resuming run"
+            else:
+                run_state_str = "Syncing run"
             run_name = self._get_run_name()
             wandb.termlog(
                 "{} {}".format(run_state_str, click.style(run_name, fg="yellow"))
             )
             self._display_run()
         print("")
+        if self._backend and not self._settings._offline:
+            self._run_status_checker = RunStatusChecker(self._backend.interface)
         self._console_start()
 
+    def _pusher_print_status(self, progress, prefix=True, done=False):
+        if self._settings._offline:
+            return
+        spinner_states = ["-", "\\", "|", "/"]
+        line = " %.2fMB of %.2fMB uploaded (%.2fMB deduped)\r" % (
+            progress.uploaded_bytes / 1048576.0,
+            progress.total_bytes / 1048576.0,
+            progress.deduped_bytes / 1048576.0,
+        )
+        line = spinner_states[self._progress_step % 4] + line
+        self._progress_step += 1
+        wandb.termlog(line, newline=False, prefix=prefix)
+
+        if done:
+            dedupe_fraction = (
+                progress.deduped_bytes / float(progress.total_bytes)
+                if progress.total_bytes > 0
+                else 0
+            )
+            if dedupe_fraction > 0.01:
+                wandb.termlog(
+                    "W&B sync reduced upload amount by %.1f%%             "
+                    % (dedupe_fraction * 100),
+                    prefix=prefix,
+                )
+            # clear progress line.
+            wandb.termlog(" " * 79, prefix=prefix)
+
+    def _on_finish_progress(self, progress, done=None):
+        self._pusher_print_status(progress, done=done)
+
+    def _wait_for_finish(self):
+        ret = None
+        while True:
+            ret = self._backend.interface.communicate_poll_exit()
+            logger.info("got exit ret: %s", ret)
+
+            done = ret.response.poll_exit_response.done
+            pusher_stats = ret.response.poll_exit_response.pusher_stats
+            if pusher_stats:
+                self._on_finish_progress(pusher_stats, done)
+            if done:
+                break
+            time.sleep(2)
+        return ret
+
     def _on_finish(self):
+        trigger.call("on_finished")
+
+        if self._run_status_checker:
+            self._run_status_checker.stop()
+
         # make sure all uncommitted history is flushed
         self.history._flush()
 
-        ret = self._backend.interface.send_exit_sync(self._exit_code, timeout=60)
-        logger.info("got exit ret: %s", ret)
-        if ret is None:
-            print("Problem syncing data")
-            os._exit(1)
-
-        #  TODO: close the logging file handler
         self._console_stop()
+        print("")
+        pid = self._backend._internal_pid
+        wandb.termlog("Waiting for W&B process to finish, PID {}".format(pid))
+        if not self._exit_code:
+            wandb.termlog("Program ended successfully.")
+        else:
+            msg = "Program failed with code {}. ".format(self._exit_code)
+            if not self._settings._offline:
+                msg += " Press ctrl-c to abort syncing."
+            wandb.termlog(msg)
+
+        # TODO: we need to handle catastrophic failure better
+        # some tests were timing out on sending exit for reasons not clear to me
+        self._backend.interface.publish_exit(self._exit_code)
+
+        # Wait for data to be synced
+        ret = self._wait_for_finish()
+
+        self._poll_exit_response = ret.response.poll_exit_response
+
+        ret = self._backend.interface.communicate_summary()
+        self._final_summary = proto_util.dict_from_proto_list(ret.item)
+
+        ret = self._backend.interface.communicate_sampled_history()
+        d = {item.key: item.values_float or item.values_int for item in ret.item}
+        self._sampled_history = d
+
         self._backend.cleanup()
 
+        if self._run_status_checker:
+            self._run_status_checker.join()
+
     def _on_final(self):
         # check for warnings and errors, show log file locations
         # if self._run_obj:
@@ -798,15 +1151,79 @@ class RunManaged(Run):
                     self._settings.log_internal
                 )
             )
+        self._show_summary()
+        self._show_history()
+        self._show_files()
+
         if self._run_obj:
             run_url = self._get_run_url()
             run_name = self._get_run_name()
             wandb.termlog(
-                "Synced {}: {}".format(
+                "\nSynced {}: {}".format(
                     click.style(run_name, fg="yellow"), click.style(run_url, fg="blue")
                 )
             )
 
+        if self._settings._offline:
+            wandb.termlog("You can sync this run to the cloud by running:")
+            wandb.termlog(
+                click.style(
+                    "wandb sync {}".format(self._settings._sync_dir), fg="yellow"
+                )
+            )
+
+    def _show_summary(self):
+        if self._final_summary:
+            logger.info("rendering summary")
+            wandb.termlog("Run summary:")
+            max_len = max([len(k) for k in self._final_summary.keys()])
+            format_str = "  {:>%s} {}" % max_len
+            for k, v in iteritems(self._final_summary):
+                # arrays etc. might be too large. for now we just don't print them
+                if isinstance(v, string_types):
+                    if len(v) >= 20:
+                        v = v[:20] + "..."
+                    wandb.termlog(format_str.format(k, v))
+                elif isinstance(v, numbers.Number):
+                    wandb.termlog(format_str.format(k, v))
+
+    def _show_history(self):
+        if not self._sampled_history:
+            return
+
+        # Only print sparklines if the terminal is utf-8
+        # In some python 2.7 tests sys.stdout is a 'cStringIO.StringO' object
+        #   which doesn't have the attribute 'encoding'
+        encoding = getattr(sys.stdout, "encoding", None)
+        if not encoding or encoding.upper() not in ("UTF_8", "UTF-8",):
+            return
+
+        logger.info("rendering history")
+        wandb.termlog("Run history:")
+        max_len = max([len(k) for k in self._sampled_history])
+        for key in self._sampled_history:
+            vals = wandb.util.downsample(self._sampled_history[key], 40)
+            if any((not isinstance(v, numbers.Number) for v in vals)):
+                continue
+            line = sparkline.sparkify(vals)
+            format_str = u"  {:>%s} {}" % max_len
+            wandb.termlog(format_str.format(key, line))
+
+    def _show_files(self):
+        if not self._poll_exit_response or not self._poll_exit_response.file_counts:
+            return
+        if self._settings._offline:
+            return
+        logger.info("logging synced files")
+        wandb.termlog(
+            "Synced {} W&B file(s), {} media file(s), {} artifact file(s) and {} other file(s)".format(  # noqa:E501
+                self._poll_exit_response.file_counts.wandb_count,
+                self._poll_exit_response.file_counts.media_count,
+                self._poll_exit_response.file_counts.artifact_count,
+                self._poll_exit_response.file_counts.other_count,
+            )
+        )
+
     def _save_job_spec(self):
         envdict = dict(python="python3.6", requirements=[],)
         varsdict = {"WANDB_DISABLE_CODE": "True"}
@@ -832,7 +1249,7 @@ class RunManaged(Run):
         }
 
         s = json.dumps(job_spec, indent=4)
-        spec_filename = "wandb-jobspec.json"
+        spec_filename = filenames.JOBSPEC_FNAME
         with open(spec_filename, "w") as f:
             print(s, file=f)
         self.save(spec_filename)
@@ -865,8 +1282,6 @@ class RunManaged(Run):
 
         if isinstance(artifact_or_name, str):
             name = artifact_or_name
-            if type is None:
-                raise ValueError("type required")
             public_api = public.Api(
                 {"entity": r.entity, "project": r.project, "run": self.id}
             )
@@ -881,13 +1296,13 @@ class RunManaged(Run):
             return artifact
         else:
             artifact = artifact_or_name
-            if type is not None:
-                raise ValueError("cannot specify type when passing Artifact object")
-            if isinstance(aliases, str):
+            if aliases is None:
+                aliases = []
+            elif isinstance(aliases, str):
                 aliases = [aliases]
             if isinstance(artifact_or_name, wandb.Artifact):
                 artifact.finalize()
-                self._backend.interface.send_artifact(
+                self._backend.interface.publish_artifact(
                     self, artifact, aliases, is_user_created=True, use_after_commit=True
                 )
                 return artifact
@@ -949,7 +1364,7 @@ class RunManaged(Run):
         if isinstance(aliases, str):
             aliases = [aliases]
         artifact.finalize()
-        self._backend.interface.send_artifact(self, artifact, aliases)
+        self._backend.interface.publish_artifact(self, artifact, aliases)
         return artifact
 
     def _set_console(self, use_redirect, stdout_slave_fd, stderr_slave_fd):
@@ -964,3 +1379,31 @@ class RunManaged(Run):
         exit_code = 0 if exc_type is None else 1
         self.join(exit_code)
         return exc_type is None
+
+
+def huggingface_version():
+    if "transformers" in sys.modules:
+        trans = wandb.util.get_module("transformers")
+        if hasattr(trans, "__version__"):
+            return trans.__version__
+    return None
+
+
+class WriteSerializingFile(object):
+    """Wrapper for a file object that serializes writes.
+    """
+
+    def __init__(self, f):
+        self.lock = threading.Lock()
+        self.f = f
+
+    def write(self, *args, **kargs):
+        self.lock.acquire()
+        try:
+            self.f.write(*args, **kargs)
+            self.f.flush()
+        finally:
+            self.lock.release()
+
+    def close(self):
+        self.f.close()
