@@ -43,15 +43,13 @@ from wandb.env import error_reporting_enabled
 import wandb
 from wandb.old.core import wandb_dir
 from wandb.errors.error import CommError
-from wandb.internal.git_repo import GitRepo
+from wandb.lib.git import GitRepo
 from wandb import env
 
 logger = logging.getLogger(__name__)
 _not_importable = set()
 
 
-OUTPUT_FNAME = 'output.log'
-DIFF_FNAME = 'diff.patch'
 
 IS_GIT = os.path.exists(
     os.path.join(os.path.dirname(__file__), '..', '.git'))
@@ -453,29 +451,6 @@ def parse_tfjob_config():
         return False
 
 
-def parse_sm_config():
-    """Attempts to parse SageMaker configuration returning False if it can't find it"""
-    sagemaker_config = "/opt/ml/input/config/hyperparameters.json"
-    resource_config = "/opt/ml/input/config/resourceconfig.json"
-    if os.path.exists(sagemaker_config) and os.path.exists(resource_config):
-        conf = {}
-        conf["sagemaker_training_job_name"] = os.getenv('TRAINING_JOB_NAME')
-        # Hyper-parameter searchs quote configs...
-        for k, v in six.iteritems(json.load(open(sagemaker_config))):
-            cast = v.strip('"')
-            if os.getenv("WANDB_API_KEY") is None and k == "wandb_api_key":
-                os.environ["WANDB_API_KEY"] = cast
-            else:
-                if re.match(r'^[-\d]+$', cast):
-                    cast = int(cast)
-                elif re.match(r'^[-.\d]+$', cast):
-                    cast = float(cast)
-                conf[k] = cast
-        return conf
-    else:
-        return False
-
-
 class WandBJSONEncoder(json.JSONEncoder):
     """A JSON Encoder that handles some extra types."""
 
@@ -707,14 +682,10 @@ def get_log_file_path():
     It would probably be better if this pointed to a log file in a
     run directory.
     """
-    # return wandb.GLOBAL_LOG_FNAME # TODO(jhr): refactor
-    return "TODO_refactor.txt"
-
-
-def is_wandb_file(name):
-    #return name.startswith('wandb') or name == wandb_config.FNAME or name == "requirements.txt" or name == OUTPUT_FNAME or name == DIFF_FNAME
-    # FIXME(jhr): need fix
-    return None
+    # TODO(jhr, cvp): refactor
+    if wandb.run:
+        return wandb.run._settings.log_internal
+    return os.path.join("wandb", "debug-internal.log")
 
 
 def docker_image_regex(image):
@@ -963,3 +934,48 @@ def to_native_slash_path(path):
 def bytes_to_hex(bytestr):
     # Works in python2 / python3
     return codecs.getencoder('hex')(bytestr)[0].decode('ascii')
+
+
+class ImportMetaHook():
+    def __init__(self):
+        self.modules = {}
+        self.on_import = {}
+
+    def add(self, fullname, on_import):
+        self.on_import.setdefault(fullname, []).append(on_import)
+
+    def install(self):
+        sys.meta_path.insert(0, self)
+
+    def uninstall(self):
+        sys.meta_path.remove(self)
+
+    def find_module(self, fullname, path=None):
+        if fullname in self.on_import:
+            return self
+
+    def load_module(self, fullname):
+        self.uninstall()
+        mod = importlib.import_module(fullname)
+        self.install()
+        self.modules[fullname] = mod
+        on_imports = self.on_import.get(fullname)
+        if on_imports:
+            for f in on_imports:
+                f()
+        return mod
+
+    def get_modules(self):
+        return tuple(self.modules)
+
+    def get_module(self, module):
+        return self.modules[module]
+
+_import_hook = None
+
+def add_import_hook(fullname, on_import):
+    global _import_hook
+    if _import_hook is None:
+        _import_hook = ImportMetaHook()
+        _import_hook.install()
+    _import_hook.add(fullname, on_import)
